#include <qstring.h>
#include <string.h>
#include <stdio.h>

#include "cline.h"
#include "ddplot.h"


void CLineHelp()
{
  printf("[ ddplot 2.3 ] - program for plotting differential displacement maps between atoms\n");
  printf("in crystalline atomic blocks. The arrows correspond to relative displacements between\n");
  printf("the connected atoms in the direction perpendicular to the plane of the screen (screw\n");
  printf("displacements) or in the plane of the screen (edge displacements).\n\n");
  printf("Calling:     ddplot [options] [plot file]\n\n");
  printf("Options:\n");
  printf("  -f file    reads the custom settings from [file]. By default, the settings\n");
  printf("             is read from file .ddplot located in your HOME directory. When using\n");
  printf("             more different settings, use -f to specify which settings should \n");
  printf("             be adopted for the current session.\n\n"); 
  printf("  -h         this help\n");
}


bool InterpretArgs( int argc, char **argv ) 
{
  QString arg;
  int i;
  bool ok=true,found;

  for (i=1; i<argc; i++) {
    arg = argv[i];
    found = false;

    // debugging information during the start of the program
    if (arg=="-debug") {
      STARTUP_DEBUG = ON;
      continue;
    }

    // load settings file instead of .ddplot
    if (arg=="-f" && i+1<argc) {
      SETTINGS_FILE = argv[++i];
      if (STARTUP_DEBUG)
        printf("User settings read from %s\n",SETTINGS_FILE.ascii());
      continue;
    }

    // command-line help
    if (arg=="-h") {
      CLineHelp();
      return false;
    }

    // last argument is the name of the plot file
    if (i+1>=argc) {
      PLOT_FILE = arg;
      if (STARTUP_DEBUG)
        printf("Plot file read from %s\n",PLOT_FILE.ascii());  
    }
  }

  return ok;
}
